import 'package:tencent_im_sdk_plugin/enum/message_priority.dart';
import 'package:tencent_im_sdk_plugin/models/v2_tim_event_callback.dart';
import 'package:tencent_im_sdk_plugin/models/v2_tim_recv_group_custom_message.dart';
import 'package:tiw_flutter/teduboad_callbackhandler.dart';
import 'package:tiw_flutter/teduboard_controller.dart';
import 'package:tiw_flutter/teduboard_def.dart';
import 'package:tiw_flutter/teduboard_view.dart';
import 'package:tiw_im_plugin_example/im_manager.dart';

const String LOG_KEY = '[BoardManager]';

class BoardManager implements TeduBoardCommonCallbacksHandler,TeduBoardProcessCallbacksHandler, TeduBoardGraffitiCallbacksHandler, TeduBoardOperationCallbacksHandler, TeduBoardFileOperationCallbacksHandler{
  String state = 'loading';
  TEduBoardController _boardController;
  TeduBoardView _boardView;
  InnerMsgListener _innerMsgListener;
  static BoardManager _sInstance;


  BoardManager._internal() {
    _innerMsgListener = new InnerMsgListener(this);
    IMManager.getInstance().addIMListener(_innerMsgListener);
  }
  
  static BoardManager getInstance() {
    if (_sInstance == null) {
      _sInstance = new BoardManager._internal();
  }
  return _sInstance;
  }

  //board 初始化
  TeduBoardView initBoard(
      TEduBoardAuthParam authParam, int roomId, TEduBoardInitParam initParam) {
    _boardView = new TeduBoardView(
      authParam: authParam,
      roomId: roomId,
      initParam: initParam,
    );
    _boardView.addCommonCallbacksHandler(this);
    _boardView.addFileOperationCallbacksHandler(this);
    _boardView.addGraffitiCallbacksHandler(this);
    _boardView.addOperationCallbacksHandler(this);
    _boardView.addProcessCallbacksHandler(this);
    
    return _boardView;
  }

  TEduBoardController getBoardController() {
    return _boardController;
  }

  //注销board，注销后board功能不可用
  void unInit() {
    if (_boardController != null) {
      _boardController.unInit();
      _boardController = null;
    }
    if (_innerMsgListener != null) {
      _innerMsgListener = null;
    }
    _sInstance = null;
    print("$LOG_KEY unInit");
  }

  ///
  /// 通用事件回调
  ///
  @override
  void onTEBError(TEduBoardErrorCode code, String msg) {
    print('$LOG_KEY onTEBError, code: $code, msg: $msg');
  }

  @override
  void onTEBWarning(TEduBoardWarningCode code, String msg) {
    // TODO: implement onTEBWarning
    print('$LOG_KEY onTEBWarning, code: $code, msg: $msg');
  }

  ///
  /// 白板基本流程回调
  ///
  @override
  void onTEBHistroyDataSyncCompleted() {
    print('$LOG_KEY onTEBHistroyDataSyncCompleted');
  }

  @override
  void onTEBInit(TEduBoardController boardController) {
    print('$LOG_KEY onTEBInit');
    _boardController = boardController;
  }

  @override
  void onTEBRedoStatusChanged(bool canRedo) {
    print('$LOG_KEY onTEBRedoStatusChanged, canRedo: $canRedo');
  }

  @override
  void onTEBSyncData(String data) {
    print('$LOG_KEY onTEBSyncData, data: $data');
    IMManager.getInstance().getInnerIM().getMessageManager().sendCustomMessage(
        data: data,
        receiver: null,
        groupID: '${_boardView.roomId}',
        priority: MessagePriority.V2TIM_PRIORITY_HIGH);
  }

  @override
  void onTEBUndoStatusChanged(bool canUndo) {
    print('$LOG_KEY onTEBUndoStatusChanged, canRedo: $canUndo');
  }

  ///
  /// 白板操作回调
  ///
  @override
  void onTEBAddBoard(List boardIds, String fileId) {
    print('$LOG_KEY onTEBAddBoard, boardIds: $boardIds, fileId: $fileId');
  }

  @override
  void onTEBDeleteBoard(List boardIds, String fileId) {
    print('$LOG_KEY onTEBDeleteBoard, boardIds: $boardIds, fileId: $fileId');
  }

  @override
  void onTEBGotoBoard(String boardId, String fileId) {
    print('$LOG_KEY onTEBGotoBoard, boardIds: $boardId, fileId: $fileId');
  }

  @override
  void onTEBGotoStep(int currentStep, int totalStep) {
    // TODO: implement onTEBGotoStep
    print(
        '$LOG_KEY onTEBGotoStep, currentStep: $currentStep, totalStep: $totalStep');
  }

  @override
  void onTEBRectSelected() {
    print('$LOG_KEY onTEBRectSelected');
  }

  @override
  void onTEBRefresh() {
    print('$LOG_KEY onTEBRefresh');
  }

  @override
  void onTEBSnapshot(String path, TEduBoardErrorCode code, String msg) {
    print('$LOG_KEY onTEBSnapshot, path: $path, code: $code, msg: $msg');
  }

  ///
  /// 涂鸦功能回调
  ///
  @override
  void onTEBAddElement(String elementId, String url) {
    print('$LOG_KEY onTEBAddElement, elementId: $elementId, url: $url');
  }

  @override
  void onTEBBackgroundH5StatusChanged(String boardId, String url, TEduBoardBackgroundH5Status status) {
    print('$LOG_KEY onTEBBackgroundH5StatusChanged, boardId: $boardId, url: $url, status: $status');
  }

  @override
  void onTEBImageStatusChanged(String boardId, String url, TEduBoardImageStatus status) {
    print('$LOG_KEY onTEBImageStatusChanged, boardId: $boardId, url: $url, status: $status');
  }

  @override
  void onTEBRemoveElement(List elementIds) {
    print('$LOG_KEY onTEBRemoveElement, elementIds: $elementIds');
  }

  @override
  void onTEBSetBackgroundImage(String url) {
    print('$LOG_KEY onTEBSetBackgroundImage, url: $url');
  }

  ///
  ///文件操作回调
  ///
  @override
  void onTEBAddImagesFile(String fileId) {
    print('$LOG_KEY onTEBAddImagesFile, fileId: $fileId');
  }

  @override
  void onTEBAddTranscodeFile(String fileId) {
    print('$LOG_KEY onTEBAddTranscodeFile, fileId: $fileId');
  }

  @override
  void onTEBAudioStatusChanged(String elementId, TEduBoardAudioStatus status,
      num progress, num duration) {
    print('$LOG_KEY onTEBAudioStatusChanged, elementId: $elementId, status: $status, progress: $progress, duration: $duration');
  }

  @override
  void onTEBDeleteFile(String fileId) {
    print('$LOG_KEY onTEBDeleteFile, fileId: $fileId');
  }

  @override
  void onTEBFileTranscodeProgress(TEduBoardTranscodeFileResult result,
      String path, String errorCode, String errorMsg) {

    print('$LOG_KEY onTEBFileTranscodeProgress, result: { taskID: ${result.taskId}, status: ${result.status}, progress: ${result.progress}, title: ${result.title}, resolution: ${result.resolution}, pages: ${result.pages}, url: ${result.url}, thumbnailResolution: ${result.thumbnailResolution}, thumbnailUrl:${result.thumbnailUrl} }, path: $path, errorCode: $errorCode, errorMsg: $errorMsg');
    
    //转码完成后添加
    if (result.status == TEduBoardFileTranscodeStatus.TEDU_BOARD_FILE_TRANSCODE_FINISHED) {
      _boardController.addTranscodeFile(result, true);
    }
  }

  @override
  void onTEBFileUploadProgress(String path, int currentBytes, int totalBytes,
      num uploadSpeed, num percent) {
    print('$LOG_KEY onTEBFileUploadProgress, path: $path, currentBytes: $currentBytes, totalBytes: $totalBytes, uploadSpeed: $uploadSpeed, percent: $percent');
  }

  @override
  void onTEBFileUploadStatus(String path, TEduBoardUploadStatus status,
      int errorCode, String errorMsg) {
    print('$LOG_KEY onTEBFileUploadStatus, path: $path, status: $status, errorCode: $errorCode, errorMsg: $errorMsg');
  }

  @override
  void onTEBH5FileStatusChanged(String fileId, TEduBoardH5FileStatus status) {
    print('$LOG_KEY onTEBH5FileStatusChanged, fileId: $fileId, status: $status');
  }

  @override
  void onTEBH5PPTStatusChanged(
      String fileId, TEduBoardH5PPTStatus status, String message) {
    print('$LOG_KEY onTEBH5PPTStatusChanged, fileId: $fileId, status: $status, message: $message');
  }

  @override
  void onTEBSwitchFile(String fileId) {
    print('$LOG_KEY onTEBSwitchFile, fileId: $fileId');
  }

  @override
  void onTEBVideoStatusChanged(String fileId, TEduBoardVideoStatus status,
      num progress, num duration) {
    print('$LOG_KEY onTEBVideoStatusChanged, fileId: $fileId, status: $status, progress: $progress, duration: $duration');
  }

}

class InnerMsgListener implements IMMsgListener {

  BoardManager _boardManager;

  InnerMsgListener(BoardManager boardManager) {
    _boardManager = boardManager;
  }

  @override
  void receiveGroupMsg(data) {
    print("$LOG_KEY receiveGroupMsg");
    // _boardManager._boardController.addSyncData(data);
  }

  @override
  void receiveAdvancedMsg(data) {
    // TODO: implement receiveAdvancedMsg
    print("$LOG_KEY receiveAdvancedMsg");
  }

  @override
  void receiveSimpleMsg(data) {
    // TODO: implement receiveSimpleMsg
    print("$LOG_KEY receiveSimpleMsg");
    V2TimEventCallback<V2TimRecvGroupCustomMessage> msg = data;
    Map<String, dynamic> dataMap = msg.toJson();
    dynamic customData = dataMap['data'];
    _boardManager.getBoardController().addSyncData(customData['customData']);
  }
}
