import 'dart:io';
import 'dart:math';

import 'package:file_picker/file_picker.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:tiw_flutter/tiw_flutter.dart';
import 'board_manager.dart';
import 'package:tiw_flutter/teduboard_def.dart';
import 'package:image_pickers/image_pickers.dart';
import 'package:webview_flutter/webview_flutter.dart';

//白板页面
class BoardPage extends StatefulWidget {
  ///授权参数
  final TEduBoardAuthParam authParam;

  /// 课堂 ID
  final int roomId;

  ///可选参数，指定用于初始化白板的一系列属性值
  ///@warning 使用腾讯云 IMSDK 进行实时数据同步时，只支持一个白板实例，创建多个白板实例可能导致涂鸦状态异常
  final TEduBoardInitParam initParam = new TEduBoardInitParam(ratio: '1:1',
      androidPlatformViewMod: AndroidPlatformViewMod.HYBRID_COMPOSITION);

  BoardPage({
    Key key,
    @required this.authParam,
    @required this.roomId,
  }) : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _BoardPageState();
  }
}

class _BoardPageState extends State<BoardPage> {
  BoardManager manager = BoardManager.getInstance();

  //菜单栏是否展开
  bool _showMenu = false;

  //白板
  TeduBoardView _boardView;

  ///数据同步
  bool _syncData = true;

  ///涂鸦开关
  bool _diyOpen = true;

  ///元素ID，只保存最后一个
  String _elementId;

  ///元素ID，只保存最后一个
  String _fileId;

  bool _audioPlaying = false;

  ///音视频播放中
  bool _videoPlaying = false;
  bool _syncVideoStatusEnable = false;
  bool _startSyncVideoStatus = false;
  bool _showVideoControl = false;

  double _volume = 0.5;

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      home: Scaffold(
        appBar: AppBar(
          title: const Text('TIW example app'),
          actions: <Widget>[
            IconButton(
              icon: Icon(Icons.menu),
              onPressed: () {
                //点击收起/展开菜单蒙层
                setState(() {
                  _showMenu = !_showMenu;
                });
              },
            )
          ],
        ),
        body: Container(
          child: _buildBodyView(),
          color: Colors.green,
        ),
      ),
    );
  }

  Widget _buildBodyView() {
    return ConstrainedBox(
      constraints: BoxConstraints.expand(),
      child: Stack(
        alignment: AlignmentDirectional.topEnd,
        children: _showMenu
            ? <Widget>[_buildBoardView(), _buildMenuMask()]
            : <Widget>[_buildBoardView()],
      ),
    );
  }

  Widget _buildBoardView() {
    if (_boardView == null) {
      _boardView =
          manager.initBoard(widget.authParam, widget.roomId, widget.initParam);
    }
    return Positioned(
        top: 50,
        width: MediaQuery.of(context).size.width,
        height: MediaQuery.of(context).size.width,
        child: _boardView);
  }

  Widget _buildMenuMask() {
    ListView _menuList = ListView(
      scrollDirection: Axis.vertical,
      children: <Widget>[
        Container(
          padding: EdgeInsets.fromLTRB(0, 10, 0, 10),
          child: Text('Element'),
        ),
        Row(
            mainAxisSize: MainAxisSize.max,
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            children: [
              OutlinedButton(
                onPressed: () async {
                  String result = await _addImageElementFromPath();
                  setState(() {
                    _elementId = result;
                  });
                },
                child: Text('本地图片元素'),
              ),
              OutlinedButton(
                onPressed: () async {
                  String result = await _addImageElementFromURL();
                  setState(() {
                    _elementId = result;
                  });
                },
                child: Text('url图片元素'),
              ),
            ]),
        Row(
            mainAxisSize: MainAxisSize.max,
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            children: [
              OutlinedButton(
                onPressed: () {
                  manager.getBoardController().removeElement(_elementId);
                  setState(() {
                    _elementId = null;
                  });
                },
                child: Text('删除元素'),
              ),
              Text('Id: ${_elementId ?? 'null'}'),
            ]),
        Row(
            mainAxisSize: MainAxisSize.max,
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            children: [
              OutlinedButton(
                onPressed: () {
                  if (_audioPlaying) {
                    manager.getBoardController().pauseAudio(_elementId);
                  } else {
                    manager.getBoardController().playAudio(_elementId);
                  }
                  setState(() {
                    _audioPlaying = !_audioPlaying;
                  });
                },
                child: Text(_audioPlaying ? '暂停' : '播放'),
              ),
              OutlinedButton(
                onPressed: () async {
                  String result = await _addAudioElement();
                  setState(() {
                    _elementId = result;
                  });
                },
                child: Text('音频url元素'),
              ),
            ]),
        Row(
            mainAxisSize: MainAxisSize.max,
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            children: [
              OutlinedButton(
                onPressed: () {
                  manager.getBoardController().seekAudio(_elementId, 10);
                },
                child: Text('seek'),
              ),
              OutlinedButton(
                onPressed: () {
                  _setVoluem();
                },
                child: Text('音量'),
              ),
            ]),
        Container(
          padding: EdgeInsets.fromLTRB(0, 10, 0, 10),
          child: Text('视频'),
        ),
        Row(
            mainAxisSize: MainAxisSize.max,
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            children: [
              OutlinedButton(
                onPressed: () async {
                  _addVideoFile();
                },
                child: Text('添加视频\ncrash!!'),
              ),
              OutlinedButton(
                onPressed: () {
                  manager
                      .getBoardController()
                      .showVideoControl(!_showVideoControl);
                  setState(() {
                    _showVideoControl = !_showVideoControl;
                  });
                },
                child: Text('视频控制栏'),
              ),
            ]),
        Row(
            mainAxisSize: MainAxisSize.max,
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            children: [
              OutlinedButton(
                onPressed: () {
                  if (_videoPlaying) {
                    manager.getBoardController().pauseVideo();
                  } else {
                    manager.getBoardController().playVideo();
                  }
                  setState(() {
                    _videoPlaying = !_videoPlaying;
                  });
                },
                child: Text(_videoPlaying ? '暂停' : '播放'),
              ),
              OutlinedButton(
                onPressed: () {
                  manager.getBoardController().seekVideo(10);
                },
                child: Text('seek'),
              ),
            ]),
        Row(
            mainAxisSize: MainAxisSize.max,
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            children: [
              OutlinedButton(
                onPressed: () {
                  manager.getBoardController().setSyncVideoStatusEnable(true);
                },
                child: Text('SyncEnable'),
              ),
              OutlinedButton(
                onPressed: () {
                  manager.getBoardController().startSyncVideoStatus(3);
                },
                child: Text('StartSync'),
              ),
            ]),
        //涂鸦操作相关
        Container(
          padding: EdgeInsets.fromLTRB(0, 10, 0, 10),
          child: Text('涂鸦'),
        ),
        Row(
            mainAxisSize: MainAxisSize.max,
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            children: [
              OutlinedButton(
                onPressed: () {
                  _printDIYInfo();
                },
                child: Text('print涂鸦信息'),
              ),
              OutlinedButton(
                onPressed: () {
                  manager.getBoardController().reset();
                },
                child: Text('重置'),
              ),
            ]),
        Row(
            mainAxisSize: MainAxisSize.max,
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            children: [
              OutlinedButton(
                onPressed: () {
                  manager.getBoardController().setDrawEnable(!_diyOpen);
                  setState(() {
                    _diyOpen = !_diyOpen;
                  });
                },
                child: Text('涂鸦开关 $_diyOpen'),
              ),
              OutlinedButton(
                onPressed: () {
                  manager.getBoardController().setDataSyncEnable(!_syncData);
                  setState(() {
                    _syncData = !_syncData;
                  });
                },
                child: Text('数据同步 $_syncData'),
              )
            ]),
        Row(
            mainAxisSize: MainAxisSize.max,
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            children: [
              OutlinedButton(
                onPressed: () {
                  manager.getBoardController().clearDraws();
                },
                child: Text('清空涂鸦'),
              ),
              OutlinedButton(
                onPressed: () {
                  manager.getBoardController().clear();
                },
                child: Text('清空涂鸦与背景'),
              )
            ]),
        Row(
            mainAxisSize: MainAxisSize.max,
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            children: [
              OutlinedButton(
                onPressed: () {
                  manager
                      .getBoardController()
                      .setGlobalBackgroundColor(0xffff1103);
                },
                child: Text('全局背景颜色'),
              ),
              OutlinedButton(
                onPressed: () {
                  manager.getBoardController().setBackgroundColor(0x81fcba03);
                },
                child: Text('背景颜色'),
              )
            ]),
        OutlinedButton(
          onPressed: () {
            manager
                .getBoardController()
                .setToolType(TEduBoardToolType.TEDU_BOARD_TOOL_TYPE_TEXT);
          },
          child: Text('选择工具'),
        ),
        Row(
            mainAxisSize: MainAxisSize.max,
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            children: [
              OutlinedButton(
                onPressed: () {
                  TEduBoardLineStyle style = TEduBoardLineStyle(
                      startArrowType:
                          TEduBoardArrowType.TEDU_BOARD_ARROW_TYPE_PLACEHOLDER,
                      endArrowType:
                          TEduBoardArrowType.TEDU_BOARD_ARROW_TYPE_NORMAL);
                  manager.getBoardController().setLineStyle(style);
                },
                child: Text('直线样式'),
              ),
              OutlinedButton(
                onPressed: () {
                  manager.getBoardController().setTextStyle(
                      TEduBoardTextStyle.TEDU_BOARD_TEXT_STYLE_BOLD_ITALIC);
                },
                child: Text('文本样式'),
              ),
            ]),
        Row(
            mainAxisSize: MainAxisSize.max,
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            children: [
              OutlinedButton(
                onPressed: () {
                  manager.getBoardController().setBrushColor(0xfffc4403);
                },
                child: Text('画笔颜色'),
              ),
              OutlinedButton(
                onPressed: () {
                  manager.getBoardController().setBrushThin(100);
                },
                child: Text('画笔粗细'),
              ),
            ]),
        Row(
            mainAxisSize: MainAxisSize.max,
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            children: [
              OutlinedButton(
                onPressed: () {
                  manager.getBoardController().setTextSize(20);
                },
                child: Text('字体大小'),
              ),
              OutlinedButton(
                onPressed: () {
                  manager.getBoardController().setTextColor(0xff1255ff);
                },
                child: Text('字体颜色'),
              ),
            ]),
        Row(
            mainAxisSize: MainAxisSize.max,
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            children: [
              OutlinedButton(
                onPressed: () {
                  _setBackgroundImage();
                },
                child: Text('背景图片'),
              ),
              OutlinedButton(
                onPressed: () {
                  manager
                      .getBoardController()
                      .setBackgroundH5('https://www.qq.com/');
                },
                child: Text('H5背景'),
              ),
            ]),
        Row(
            mainAxisSize: MainAxisSize.max,
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            children: [
              OutlinedButton(
                onPressed: () {
                  manager.getBoardController().undo();
                },
                child: Text('撤回'),
              ),
              OutlinedButton(
                onPressed: () {
                  manager.getBoardController().redo();
                },
                child: Text('重做'),
              )
            ]),

        //白板操作相关
        Container(
          padding: EdgeInsets.fromLTRB(0, 10, 0, 10),
          child: Text('白板操作'),
        ),
        Row(
          mainAxisSize: MainAxisSize.max,
          mainAxisAlignment: MainAxisAlignment.spaceAround,
          children: [
            OutlinedButton(
              onPressed: () {
                manager.getBoardController().addBoardWithBackgroundImage(null);
              },
              child: Text('新增白板'),
            ),
            OutlinedButton(
              onPressed: () async {
                String currentBoard =
                    await manager.getBoardController().getCurrentBoard();
                manager.getBoardController().deleteBoard(currentBoard);
              },
              child: Text('删除当前白板'),
            ),
          ],
        ),
        Row(
          mainAxisSize: MainAxisSize.max,
          mainAxisAlignment: MainAxisAlignment.spaceAround,
          children: [
            OutlinedButton(
              onPressed: () {
                manager.getBoardController().prevStep();
              },
              child: Text('上一步'),
            ),
            OutlinedButton(
              onPressed: () async {
                manager.getBoardController().nextStep();
              },
              child: Text('下一步'),
            ),
          ],
        ),
        Row(
          mainAxisSize: MainAxisSize.max,
          mainAxisAlignment: MainAxisAlignment.spaceAround,
          children: [
            OutlinedButton(
              onPressed: () {
                manager.getBoardController().preBoardAndReset(true);
              },
              child: Text('上一页'),
            ),
            OutlinedButton(
              onPressed: () async {
                String currentBoard =
                    await manager.getBoardController().getCurrentBoard();
                manager.getBoardController().nextBoardAndReset(true);
              },
              child: Text('下一页'),
            ),
          ],
        ),
        Row(
            mainAxisSize: MainAxisSize.max,
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            children: [
              OutlinedButton(
                onPressed: () async {
                  List<String> boardList =
                      await manager.getBoardController().getBoardList();
                  manager
                      .getBoardController()
                      .gotoBoardAndReset(boardList.elementAt(0), true);
                },
                child: Text('跳转首页'),
              ),
              OutlinedButton(
                onPressed: () async {
                  String ratio =
                      await manager.getBoardController().getBoardRatio();
                  if (ratio == '1:1') {
                    ratio = '16:9';
                  } else {
                    ratio = '1:1';
                  }
                  manager.getBoardController().setBoardRatio(ratio);
                },
                child: Text('切换白板比例'),
              ),
            ]),
        Row(
            mainAxisSize: MainAxisSize.max,
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            children: [
              OutlinedButton(
                onPressed: () async {
                  int scale =
                      await manager.getBoardController().getBoardScale();
                  scale += 200;
                  scale = min(1600, scale);
                  manager.getBoardController().setBoardScale(scale);
                },
                child: Text('+缩放比例'),
              ),
              OutlinedButton(
                onPressed: () async {
                  int scale =
                      await manager.getBoardController().getBoardScale();
                  scale -= 200;
                  scale = max(100, scale);
                  manager.getBoardController().setBoardScale(scale);
                },
                child: Text('-缩放比例'),
              ),
            ]),
        //文件操作相关
        Container(
          padding: EdgeInsets.fromLTRB(0, 10, 0, 10),
          child: Text('文件操作'),
        ),
        Row(
            mainAxisSize: MainAxisSize.max,
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            children: [
              OutlinedButton(
                onPressed: () {
                  manager.getBoardController().addH5File('https://cloud.tencent.com/product/tiw');
                  manager.getBoardController().addH5File('https://cloud.tencent.com/');
                },
                child: Text('增加H5文件'),
              ),
              OutlinedButton(
                onPressed: () async {
                  String currentFile =
                      await manager.getBoardController().getCurrentFile();
                  manager.getBoardController().deleteFile(currentFile);
                },
                child: Text('删除当前文件'),
              ),
            ]),
        Row(
            mainAxisSize: MainAxisSize.max,
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            children: [
              OutlinedButton(
                onPressed: () async {
                  String currentFileId = await manager.getBoardController().getCurrentFile();
                  print('测试File接口 getCurrentFile: $currentFileId');
                  List<String> boardList = await manager.getBoardController().getFileBoardList(currentFileId);
                  print('测试File接口 getFileBoardList: $boardList');
                  List<String> thumbList = await manager.getBoardController().getThumbnailImages(currentFileId);
                  print('测试File接口 getThumbnailImages: $thumbList');
                  TEduBoardFileInfo currentFileInfo = await manager.getBoardController().getFileInfo(currentFileId);
                  print('测试File接口 getFileInfo: $currentFileInfo');
                  List<TEduBoardFileInfo> fileInfoList = await manager.getBoardController().getFileInfoList();
                  print('测试File接口 getFileInfoList: $fileInfoList');
                },
                child: Text('测试读取接口'),
              ),
              OutlinedButton(
                onPressed: () async {
                  String currentFile =
                      await manager.getBoardController().getCurrentFile();
                  List<TEduBoardFileInfo> fileInfoList = await manager.getBoardController().getFileInfoList();
                  int currentIndex = 0;
                  int index = 0;
                  fileInfoList.forEach((element) {
                    if (element.fileId == currentFile) {
                      currentIndex = index;
                    }
                    index++;
                  });
                  currentIndex = currentIndex == fileInfoList.length-1 ? 0 : currentIndex+1;
                  String targetFileId = fileInfoList.elementAt(currentIndex).fileId;
                  manager.getBoardController().switchFile(targetFileId);
                  targetFileId =
                      await manager.getBoardController().getCurrentFile();
                  print('测试File接口 switchFile: from $currentFile to $targetFileId');    
                },
                child: Text('切换文件'),
              ),
            ]),
        Row(
            mainAxisSize: MainAxisSize.max,
            mainAxisAlignment: MainAxisAlignment.start,
            children: [
              OutlinedButton(
                onPressed: () async {
                  FilePickerResult result = await FilePicker.platform.pickFiles();
                  if(result != null) {
                    String path = result.files.single.path;
                    TEduBoardTranscodeConfig config = TEduBoardTranscodeConfig(
                        minRosolution: '960x540',
                        thumbnailResolution: '200x200');
                    manager
                        .getBoardController()
                        .applyFileTranscode(path, config);
                  } else {
                    // User canceled the picker
                  }
                },
                child: Text('上传PPT/PDF/Word'),
              ),
            ]),
      ],
    );

    return Positioned(
        right: 0,
        top: 0,
        bottom: 0,
        width: 250,
        child: Container(
          color: Colors.white70,
          padding: EdgeInsets.fromLTRB(10, 20, 10, 20),
          child: _menuList,
        ));
  }

////////////////////////////白板涂鸦begin////////////////////////////
  void _printDIYInfo() async {
    bool diyOpen = await manager.getBoardController().isDrawEnable();
    bool syncOpen = await manager.getBoardController().isDataSyncEnable();

    int gColor = await manager.getBoardController().getGlobalBackgroundColor();

    int bgColor = await manager.getBoardController().getBackgroundColor();

    TEduBoardToolType toolType =
        await manager.getBoardController().getToolType();

    int textSize = await manager.getBoardController().getTextSize();
    int textColor = await manager.getBoardController().getTextColor();
    TEduBoardTextStyle textStyle =
        await manager.getBoardController().getTextStyle();

    int brushThin = await manager.getBoardController().getBrushThin();
    int brushColor = await manager.getBoardController().getBrushColor();

    TEduBoardLineStyle lineStyle =
        await manager.getBoardController().getLineStyle();

    double volume =
        await manager.getBoardController().getAudioVolume(_elementId);

    List<TEduBoardElementInfo> elementList =
        await manager.getBoardController().getBoardElementList(null);

    String info = """DIY INFO:
    isDrawEnable: ${diyOpen ?? ''},,
    isDataSyncEnable: ${syncOpen ?? ''},,
    getGlobalBackgroundColor: 0x${gColor?.toRadixString(16)},

    getBackgroundColor: 0x${bgColor?.toRadixString(16)},
    getToolType:${toolType ?? ''},

    getTextSize:${textSize ?? ''},
    getTextColor:0x${textColor?.toRadixString(16)},
    getTextStyle: ${textStyle ?? ''},

    getBrushThin:${brushThin ?? ''},
    getBrushColor:0x${brushColor?.toRadixString(16)},

    getLineStyle:${lineStyle.toString()},

    getAudioVolume:${_elementId ?? ''}, ${volume ?? ''},

    getBoardElementList: ${elementList?.toString()}
    """;
    print(info);
  }

  void _setBackgroundImage() async {
    String path = await _selectImages();
    print('_setBackgroundImage: $path');
    if (path == null || path.length <= 0) {
      return;
    }
    manager.getBoardController().setBackgroundImage(
        path, TEduBoardImageFitMode.TEDU_BOARD_IMAGE_FIT_MODE_LEFT);
  }

  Future<String> _addImageElementFromPath() async {
    String path = await _selectImages();
    print('_addElementFromPath: $path');
    if (path == null || path.length <= 0) {
      return null;
    }
    String result = await manager
        .getBoardController()
        .addElement(path, TEduBoardElementType.TEDU_BOARD_ELEMENT_IMAGE);
    print('_addElementFromPath elementId: $result');
    return result;
  }

  Future<String> _addImageElementFromURL() async {
    print('_addElementFromURL: 	http://down.qq.com/innovate/qqclass/pic1.jpg');
    String result = await manager.getBoardController().addElement(
        'http://down.qq.com/innovate/qqclass/pic1.jpg',
        TEduBoardElementType.TEDU_BOARD_ELEMENT_IMAGE);
    print('_addElementFromURL elementId: $result');
    return result;
  }

  Future<String> _addAudioElement() async {
    String result = await manager.getBoardController().addElement(
        'http://down.qq.com/innovate/audio.mp3',
        TEduBoardElementType.TEDU_BOARD_ELEMENT_AUDIO);
    print('_addVideoElement elementId: $result');
    return result;
  }

  Future<String> _addVideoFile() async {
    String result = await manager
        .getBoardController()
        .addVideoFile('http://down.qq.com/innovate/video.mp4');
    print('_addVideoFile fileid: $result');
    return result;
    // String result =  await manager
    //     .getBoardController()
    //     .addImagesFile([
    //   'http://down.qq.com/innovate/qqclass/pic1.jpg',
    //   'http://down.qq.com/innovate/qqclass/pic1.jpg'
    // ]);
    // print('_addVideoFile fileid: $result');
    // return result;
  }

  void _setVoluem() {
    _volume += 0.1;
    if (_volume >= 1) {
      _volume = 0.1;
    }
    manager.getBoardController().setAudioVolume(_elementId, _volume);
  }

////////////////////////////白板涂鸦 end////////////////////////////

////////////////////////////选照片 begin////////////////////////////
  Future<String> _selectImages() async {
    List<Media> _listImagePaths = await ImagePickers.pickerPaths(
      galleryMode: GalleryMode.image,
      showGif: true,
      selectCount: 5,
      showCamera: true,
      cropConfig: CropConfig(enableCrop: true, height: 1, width: 1),
      compressSize: 500,
      uiConfig: UIConfig(
        uiThemeColor: Color(0xffff0000),
      ),
    );
    return _listImagePaths[0].path;
  }
}

////////////////////////////选照片 end////////////////////////////
