import 'package:tencent_im_sdk_plugin/enum/log_level.dart';
import 'package:tencent_im_sdk_plugin/manager/v2_tim_manager.dart';
import 'package:tencent_im_sdk_plugin/models/v2_tim_callback.dart';
import 'package:tencent_im_sdk_plugin/models/v2_tim_value_callback.dart';
import 'package:tencent_im_sdk_plugin/tencent_im_sdk_plugin.dart';

import 'login/login_manager.dart';

class IMManager {
  static const TAG = "IMManager";
  static IMManager _sInstance;

  V2TIMManager _v2timManager;
  List<IMMsgListener> _imListenerList;

  IMManager._internal() {
    _imListenerList = [];
    _v2timManager = TencentImSDKPlugin.v2TIMManager;
    _initTIMManager();
  }

  static IMManager getInstance() {
    if (_sInstance == null) {
      _sInstance = new IMManager._internal();
    }
    return _sInstance;
  }

  void destroy(){
    _imListenerList.clear();
    _v2timManager = null;
    _sInstance = null;
  }

  V2TIMManager getInnerIM() {
    return _v2timManager;
  }

  void addIMListener(IMMsgListener listener) {
    _imListenerList.add(listener);
  }

  void removeIMListener(IMMsgListener listener) {
    _imListenerList.remove(listener);
  }

  /// =============初始化逻辑=============

  ///IMSDK 初始化
  Future<V2TimValueCallback<bool>> initSdk(Function listener) async {
    V2TimValueCallback<bool> result = await _v2timManager.initSDK(sdkAppID: SDK_APPID, loglevel: LogLevel.V2TIM_LOG_DEBUG, listener: listener);
    print(TAG + " init sdk code:${result.code} desc:${result.desc} data:${result.data}");
    return result;
  }

  ///IMSDK反初始化
  Future<V2TimCallback> unInitSDK() async {
    V2TimCallback result = await _v2timManager.unInitSDK();
    print(TAG + " unInitSDK code:${result.code} desc:${result.desc}");
    return result;
  }

  ///IMSDK登陆
  Future<V2TimCallback> login(String userId, String userSig) async {
    V2TimCallback result = await _v2timManager.login(userID: userId, userSig: userSig);
    print(TAG + " login code:${result.code} desc:${result.desc}");
    return result;
  }

  ///IMSDM注销
  Future<V2TimCallback> logout() async{
    V2TimCallback result = await _v2timManager.logout();
    print(TAG + " logout code:${result.code} desc:${result.desc}");
    return result;
  }

  /// =============登陆信息=============
  Future<V2TimValueCallback<String>> getLoginUser() async {
    return _v2timManager.getLoginUser();
  }

  Future<V2TimValueCallback<int>> getLoginStatus() async {
    return _v2timManager.getLoginStatus();
  }

  /// =============消息接口=============
  void _initTIMManager(){
    _v2timManager.addSimpleMsgListener(listener: _simpleMsgListener);
    _v2timManager.setGroupListener(listener: _groupMsgListener);
    _v2timManager.getMessageManager().addAdvancedMsgListener(listener: _advancedMsgListener);
  }

  _simpleMsgListener(data){
    for(IMMsgListener listener in _imListenerList){
      listener.receiveSimpleMsg(data);
    }
  }
  _groupMsgListener(data){
    for(IMMsgListener listener in _imListenerList){
      listener.receiveGroupMsg(data);
    }
  }
  _advancedMsgListener(data){
    for(IMMsgListener listener in _imListenerList){
      listener.receiveAdvancedMsg(data);
    }
  }

}

class IMMsgListener{
  ///普通消息监听
  void receiveSimpleMsg(data){
  }

  ///群组消息监听
  void receiveGroupMsg(data){
  }

  ///高级消息监听
  void receiveAdvancedMsg(data){
  }
}
