import 'package:flutter/material.dart';
import '../board/board_page.dart';
import 'login_manager.dart';
import 'package:toast/toast.dart';
import 'package:tencent_im_sdk_plugin/models/v2_tim_callback.dart';
import 'package:tiw_flutter/teduboard_def.dart';

class LoginPage extends StatefulWidget{
  @override
  State<StatefulWidget> createState() {
    return _LoginPageState();
  }
}

class _LoginPageState extends State<LoginPage>{
  /// 用户id
  String userId = '';
  /// 会议id
  String meetId = '';

  String state = '';
  LoginManager _loginManager;
  final meetIdFocusNode = FocusNode();
  final userFocusNode = FocusNode();

  @override
  void initState() {
    super.initState();
    _loginManager = new LoginManager();
    _loginManager.initTIMManager();
  }

  // 提示浮层
  showToast(text) {
    Toast.show(
      text,
      context,
      duration: Toast.LENGTH_SHORT,
      gravity: Toast.CENTER,
    );
  }

  // 隐藏底部输入框
  unFocus() {
    if (meetIdFocusNode.hasFocus) {
      meetIdFocusNode.unfocus();
    } else if (userFocusNode.hasFocus) {
      userFocusNode.unfocus();
    }
  }

  enterMeeting() async {
    meetId = meetId.replaceAll(new RegExp(r"\s+\b|\b\s"), "");
    if (meetId == '') {
      showToast('请输入会议号');
      return;
    } else if (meetId == '0') {
      showToast('请输入合法的会议ID');
      return;
    } else if (meetId.toString().length > 10) {
      showToast('会议ID过长，请输入合法的会议ID');
      return;
    } else if (!new RegExp(r"[0-9]+$").hasMatch(meetId)) {
      showToast('会议ID只能为数字，请输入合法的会议ID');
      return;
    }
    userId = userId.replaceAll(new RegExp(r"\s+\b|\b\s"), "");
    if (userId == '') {
      showToast('请输入用户ID');
      return;
    } else if (!new RegExp(r"[A-Za-z0-9_]+$").hasMatch(userId)) {
      showToast('用户ID只能为数字、字母、下划线，请输入正确的用户ID');
      return;
    } else if (userId.length > 10) {
      showToast('用户ID过长，请输入合法的用户ID');
      return;
    }
    unFocus();

    V2TimCallback callback = await _loginManager.login(userId);
    if (callback.code == 0) {
      showToast('登陆成功，开始进房');
      //demo简单粗暴一点
      await _loginManager.createGroup(meetId);
      await _loginManager.quitGroup(meetId);
      V2TimCallback result = await _loginManager.joinGroup(meetId);
      if (result.code == 0 || result.code == 10013) { // 10013是已在房间中，可以直接进
        showToast('进房成功');
        TEduBoardAuthParam authParam = new TEduBoardAuthParam(SDK_APPID, _loginManager.getCurrentUid(), _loginManager.getCurrentSig());
        Navigator.push(context, MaterialPageRoute(
          builder: (context) => BoardPage(authParam: authParam, roomId: int.parse(meetId)),
          settings: RouteSettings(name: "/board", arguments: {
            "authParam": authParam,
            "roomId": meetId,
          }),
        ));
      } else {
        showToast('进房失败 code:${result.code} desc:' + result.desc);
      }
    } else {
      showToast('登陆失败 code:${callback.code} desc:' + callback.desc);
    }
  }

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      home: Scaffold(
        appBar: AppBar(
          title: const Text('Plugin example app'),
        ),
        body: GestureDetector(
          behavior: HitTestBehavior.translucent,
          onTap: () {
            if (meetIdFocusNode.hasFocus) {
              meetIdFocusNode.unfocus();
            } else if (userFocusNode.hasFocus) {
              userFocusNode.unfocus();
            }
          },
          child: Container(
            color: Color.fromRGBO(14, 25, 44, 1),
            padding: const EdgeInsets.symmetric(horizontal: 20.0),
            child: Column(
              children: <Widget>[
                Container(
                  color: Color.fromRGBO(13, 44, 91, 1),
                  margin: const EdgeInsets.only(top: 60.0),
                  padding: const EdgeInsets.only(left: 10.0, right: 10.0),
                  child: Column(
                    children: [
                      TextField(
                          style: TextStyle(color: Colors.white),
                          autofocus: false,
                          focusNode: meetIdFocusNode,
                          decoration: InputDecoration(
                            labelText: "会议号",
                            hintText: "请输入会议号",
                            labelStyle: TextStyle(color: Colors.white),
                            hintStyle: TextStyle(
                                color: Color.fromRGBO(255, 255, 255, 0.5)),
                            enabledBorder: UnderlineInputBorder(
                              borderSide: BorderSide(color: Colors.white),
                            ),
                          ),
                          keyboardType: TextInputType.number,
                          onChanged: (value) => meetId = value),
                      TextField(
                          style: TextStyle(color: Colors.white),
                          autofocus: false,
                          focusNode: userFocusNode,
                          decoration: InputDecoration(
                            labelText: "用户ID",
                            hintText: "请输入用户ID",
                            labelStyle: TextStyle(color: Colors.white),
                            hintStyle: TextStyle(
                                color: Color.fromRGBO(255, 255, 255, 0.5)),
                            border: InputBorder.none,
                          ),
                          keyboardType: TextInputType.text,
                          onChanged: (value) => this.userId = value),
                    ],
                  ),
                ),
                Padding(
                  padding: const EdgeInsets.only(top: 30.0),
                  child: Row(
                    children: <Widget>[
                      Expanded(
                        child: RaisedButton(
                          padding: EdgeInsets.all(15.0),
                          child: Text("进入会议"),
                          color: Theme.of(context).primaryColor,
                          textColor: Colors.white,
                          onPressed: enterMeeting,
                        ),
                      ),
                    ],
                  ),
                )
              ],
            ),
          ),
        ),
      ),
    );
  }

}