import 'dart:core';
import 'dart:ui';

import 'package:flutter/material.dart';

/// =============白板枚举定义=============
/// enum对应的原始值为int，通过添加placeholder令原始值对应values数组下标
/// int转enum：EnumType enumValue = Enum.vlues[num]

/// 白板工具
enum TEduBoardToolType {
  TEDU_BOARD_TOOL_TYPE_MOUSE,                  ///< 鼠标
  TEDU_BOARD_TOOL_TYPE_PEN,                    ///< 画笔
  TEDU_BOARD_TOOL_TYPE_ERASER,                 ///< 橡皮擦
  TEDU_BOARD_TOOL_TYPE_LASER,                  ///< 激光笔
  TEDU_BOARD_TOOL_TYPE_LINE,                   ///< 直线
  TEDU_BOARD_TOOL_TYPE_OVAL,                   ///< 空心椭圆
  TEDU_BOARD_TOOL_TYPE_RECT,                   ///< 空心矩形
  TEDU_BOARD_TOOL_TYPE_OVAL_SOLID,             ///< 实心椭圆
  TEDU_BOARD_TOOL_TYPE_RECT_SOLID,             ///< 实心矩形
  TEDU_BOARD_TOOL_TYPE_POINT_SELECT,           ///< 点选工具
  TEDU_BOARD_TOOL_TYPE_RECT_SELECT,           ///< 选框工具
  TEDU_BOARD_TOOL_TYPE_TEXT,                  ///< 文本工具
  TEDU_BOARD_TOOL_TYPE_ZOOM_DRAG,             ///< 缩放移动
  TEDU_BOARD_TOOL_TYPE_SQUARE,                ///<空心正方形
  TEDU_BOARD_TOOL_TYPE_SQUARE_SOLID,          ///<实心正方形
  TEDU_BOARD_TOOL_TYPE_CIRCLE,                ///<空心正圆形
  TEDU_BOARD_TOOL_TYPE_CIRCLE_SOLID,          ///<实心正圆形
  TEDU_BOARD_TOOL_TYPE_BOARD_CUSTOM_GRAPH,    ///<自定义图形，请配合addElement(TEDU_BOARD_ELEMENT_CUSTOM_GRAPH, '自定义图形URL')接口使用
}

/// 白板图片填充对齐模式
enum TEduBoardImageFitMode {
  TEDU_BOARD_IMAGE_FIT_MODE_CENTER,            ///< 以宽度或者高度为基准居中对齐等比例放大
  TEDU_BOARD_IMAGE_FIT_MODE_PLACEHOLDER1,      ///< 占位
  TEDU_BOARD_IMAGE_FIT_MODE_PLACEHOLDER2,      ///< 占位
  TEDU_BOARD_IMAGE_FIT_MODE_PLACEHOLDER3,      ///< 占位
  TEDU_BOARD_IMAGE_FIT_MODE_LEFT,              ///< 以宽度或者高度为基准左对齐等比例放大
  TEDU_BOARD_IMAGE_FIT_MODE_TOP,               ///< 以宽度或者高度为基准顶对齐等比例放大
  TEDU_BOARD_IMAGE_FIT_MODE_RIGHT,             ///< 以宽度或者高度为基准右对齐等比例放大
  TEDU_BOARD_IMAGE_FIT_MODE_BOTTOM,            ///< 以宽度或者高度为基准底对齐等比例放大  
}

/// 白板内容自适应模式
enum TEduBoardContentFitMode {
  TEDU_BOARD_CONTENT_FIT_MODE_NONE,                  ///< 不使用内容自适应，默认模式，不自动调整白板宽高比，内容等比例缩放居中显示，内容宽高 <= 白板宽高
  TEDU_BOARD_CONTENT_FIT_MODE_CENTER_INSIDE,         ///< 自动调整白板宽高比与内容一致，内容铺满白板，白板等比例缩放居中显示，白板宽高 <= 容器宽高
  TEDU_BOARD_CONTENT_FIT_MODE_CENTER_COVER,          ///< 自动调整白板宽高比与内容一致，内容铺满白板，白板等比例缩放居中显示，白板宽高 >= 容器宽高 
}

/// 白板图片状态
enum TEduBoardImageStatus {
  TEDU_BOARD_IMAGE_STATUS_PLACEHOLDER,        ///< 占位
  TEDU_BOARD_IMAGE_STATUS_LOADING,            ///< 背景图片正在加载
  TEDU_BOARD_IMAGE_STATUS_LOAD_DONE,          ///< 背景图片加载完成
  TEDU_BOARD_IMAGE_STATUS_LOAD_ABORT,         ///< 背景图片加载中断
  TEDU_BOARD_IMAGE_STATUS_LOAD_ERROR,         ///< 背景图片加载错误
  TEDU_BOARD_IMAGE_STATUS_LOAD_TIMEOUT,       ///< 背景图片加载超时
  TEDU_BOARD_IMAGE_STATUS_LOAD_CANCEL,        ///< 背景图片取消加载
  TEDU_BOARD_IMAGE_STATUS_READ_ERROR,         ///< 背景图片读取失败 
}

/// 白板文本样式
enum TEduBoardTextStyle {
  TEDU_BOARD_TEXT_STYLE_NORMAL,                ///< 常规样式
  TEDU_BOARD_TEXT_STYLE_BOLD,                  ///< 粗体样式
  TEDU_BOARD_TEXT_STYLE_ITALIC,                ///< 斜体样式
  TEDU_BOARD_TEXT_STYLE_BOLD_ITALIC,           ///< 粗斜体样式
}

/// 白板错误码（严重）
enum TEduBoardErrorCode {
  TEDU_BOARD_ERROR_NONE,                      ///< 占位
  TEDU_BOARD_ERROR_INIT,                      ///< 白板初始化失败
  TEDU_BOARD_ERROR_AUTH,                      ///< 服务鉴权失败，未购买服务或服务已过期
  TEDU_BOARD_ERROR_LOAD,                      ///< 白板加载失败
  TEDU_BOARD_ERROR_PLACEHOLDER4,              ///< 占位
  TEDU_BOARD_ERROR_TIM_INVALID,               ///< 腾讯云 IMSDK 不可用
  TEDU_BOARD_ERROR_TIM_HISTORYDATA,           ///< 同步历史数据失败
  TEDU_BOARD_ERROR_RUNTIME,                   ///< 白板运行错误
  TEDU_BOARD_ERROR_AUTH_TIMEOUT,              ///< 服务鉴权超时，有可能存在网络问题，请重试
  TEDU_BOARD_ERROR_DATA_TOO_LARGE,            ///< 传输的数据太大，请分拆成两个或多个后传递(比如，添加图片组文件时url数量太大)
  TEDU_BOARD_ERROR_PATH_INVALID,              ///< 路径非法
  TEDU_BOARD_ERROR_WRITE_ERROR,               ///< 文件写入错误  
}

/// 白板错误码（警告）
enum TEduBoardWarningCode {
  TEDU_BOARD_WARNING_NONE,                             ///< 占位
  TEDU_BOARD_WARNING_SYNC_DATA_PARSE_FAILED,           ///< 接收到其他端的同步数据解析失败
  TEDU_BOARD_WARNING_TIM_SEND_MSG_FAILED,              ///< 腾讯 IMSDK 发送消息失败
  TEDU_BOARD_WARNING_H5PPT_ALREADY_EXISTS,             ///< 当要添加的 H5PPT 已存在时抛出该警告
  TEDU_BOARD_WARNING_PLACEHOLDER4,                     ///< 占位
  TEDU_BOARD_WARNING_ILLEGAL_OPERATION,                ///< 白板历史数据加载完成之前禁止操作
  TEDU_BOARD_WARNING_H5FILE_ALREADY_EXISTS,            ///< 当要添加的 H5File 已存在时抛出该警告
  TEDU_BOARD_WARNING_VIDEO_ALREADY_EXISTS,             ///< 当要添加的视频已存在时抛出该警告
  TEDU_BOARD_WARNING_IMAGESFILE_ALREADY_EXISTS,        ///< 当要添加的图片文件已存在时抛出该警告  
}

/// 白板上传状态
enum TEduBoardUploadStatus {
  TEDU_BOARD_UPLOAD_STATUS_PLACEHOLDER,       ///< 占位
  TEDU_BOARD_UPLOAD_STATUS_SUCCEED,           ///< 上传成功
  TEDU_BOARD_UPLOAD_STATUS_FAILED,            ///< 上传失败  
}

/// H5 背景状态
enum TEduBoardBackgroundH5Status {
  TEDU_BOARD_BACKGROUND_H5_STATUS_PLACEHOLDER,      ///< 占位
  TEDU_BOARD_BACKGROUND_H5_STATUS_LOADING,          ///< H5 背景正在加载
  TEDU_BOARD_BACKGROUND_H5_STATUS_LOAD_FINISH,      ///< H5 背景加载完成  
}

/// 直线类型
enum TEduBoardLineType {
  TEDU_BOARD_LINE_TYPE_PLACEHOLDER,  ///< 占位
  TEDU_BOARD_LINE_TYPE_SOLID,        ///< 实线
  TEDU_BOARD_LINE_TYPE_DOTTED,       ///< 虚线  
}

/// 箭头类型
enum TEduBoardArrowType {
  TEDU_BOARD_ARROW_TYPE_PLACEHOLDER,  ///< 占位
  TEDU_BOARD_ARROW_TYPE_NONE,         ///< 无箭头
  TEDU_BOARD_ARROW_TYPE_NORMAL,       ///< 普通箭头
  TEDU_BOARD_ARROW_TYPE_SOLID,        ///< 实心箭头  
}

/// 椭圆绘制模式
enum TEduBoardOvalDrawMode {
  TEDU_BOARD_OVAL_DRAW_MODE_PLACEHOLDER,      ///< 占位
  TEDU_BOARD_OVAL_DRAW_MODE_FIX_STAR,         ///< 固定起始点，起始点和终点的中点为圆心
  TEDU_BOARD_OVAL_DRAW_MODE_FIX_CENTER,       ///< 固定圆心，起始点为圆心 
}

/// 文件转码状态
enum TEduBoardFileTranscodeStatus {
  TEDU_BOARD_FILE_TRANSCODE_PLACEHOLDER,       ///< 占位
  TEDU_BOARD_FILE_TRANSCODE_ERROR,             ///< 转码错误
  TEDU_BOARD_FILE_TRANSCODE_UPLOADING,         ///< 文件上传中
  TEDU_BOARD_FILE_TRANSCODE_CREATED,           ///< 发起转码任务
  TEDU_BOARD_FILE_TRANSCODE_QUEUED,            ///< 排队中
  TEDU_BOARD_FILE_TRANSCODE_PROCESSING,        ///< 转码中
  TEDU_BOARD_FILE_TRANSCODE_FINISHED,          ///< 转码完成
}

/// 视频文件状态
enum TEduBoardVideoStatus {
  TEDU_BOARD_VIDEO_STATUS_PLACEHOLDER,     ///< 占位
  TEDU_BOARD_VIDEO_STATUS_ERROR,           ///< 播放错误
  TEDU_BOARD_VIDEO_STATUS_LOADING,         ///< 加载中
  TEDU_BOARD_VIDEO_STATUS_LOADED,          ///< 加载完成
  TEDU_BOARD_VIDEO_STATUS_PLAYED,          ///< 开始播放
  TEDU_BOARD_VIDEO_STATUS_TIMEUPDATE,      ///< 时间更新
  TEDU_BOARD_VIDEO_STATUS_PAUSED,          ///< 暂停
  TEDU_BOARD_VIDEO_STATUS_SEEKED,          ///< 跳转
  TEDU_BOARD_VIDEO_STATUS_ENDED,           ///< 结束
  TEDU_BOARD_VIDEO_STATUS_WAITING,         ///< 缓冲中
  TEDU_BOARD_VIDEO_STATUS_PLAYING,         ///< 缓冲结束 
}

/// 音频状态
enum TEduBoardAudioStatus {
  TEDU_BOARD_AUDIO_STATUS_PLACEHOLDER,     ///< 占位
  TEDU_BOARD_AUDIO_STATUS_ERROR,           ///< 播放错误
  TEDU_BOARD_AUDIO_STATUS_LOADING,         ///< 加载中
  TEDU_BOARD_AUDIO_STATUS_LOADED,          ///< 加载完成
  TEDU_BOARD_AUDIO_STATUS_PLAYED,          ///< 开始播放
  TEDU_BOARD_AUDIO_STATUS_TIMEUPDATE,      ///< 时间更新
  TEDU_BOARD_AUDIO_STATUS_PAUSED,          ///< 暂停
  TEDU_BOARD_AUDIO_STATUS_SEEKED,          ///< 跳转
  TEDU_BOARD_AUDIO_STATUS_ENDED,           ///< 结束
  TEDU_BOARD_AUDIO_STATUS_WAITING,         ///< 缓冲中
  TEDU_BOARD_AUDIO_STATUS_PLAYING,         ///< 缓冲结束  
}

/// PPT 动画文件加载状态
enum TEduBoardH5PPTStatus {
  TEDU_BOARD_H5_PPT_STATUS_PLACEHOLDER,                    ///< 占位
  TEDU_BOARD_H5_PPT_STATUS_LOADING,                        ///< PPT动画文件加载中
  TEDU_BOARD_H5_PPT_STATUS_LOADED,                         ///< PPT动画文件加载完成
  TEDU_BOARD_H5_PPT_STATUS_CANCEL,                         ///< PPT动画文件取消加载 (本地PPT还没有加载完成，就切换到了其他文件)
  TEDU_BOARD_H5_PPT_STATUS_TIMEOUT,                        ///< PPT动画文件加载超时
  TEDU_BOARD_H5_PPT_STATUS_RESOURCES_LOADING_ERROR,        ///< PPT动画文件加载失败
  TEDU_BOARD_H5_PPT_STATUS_RUNTIME_ERROR,                  ///< PPT动画文件加载内部运行错误 
}

/// H5 文件状态
enum TEduBoardH5FileStatus {
  TEDU_BOARD_H5_FILE_STATUS_PLACEHOLDER,       ///< 占位
  TEDU_BOARD_H5_FILE_STATUS_LOADING,           ///< 加载中
  TEDU_BOARD_H5_FILE_STATUS_LOADED,            ///< 加载完成
}

/// 元素类型
enum TEduBoardElementType {
  TEDU_BOARD_ELEMENT_PLACEHOLDER,       ///< 占位
  TEDU_BOARD_ELEMENT_IMAGE,             //< 图片元素
  TEDU_BOARD_ELEMENT_H5,                //< 网页元素
  TEDU_BOARD_ELEMENT_CUSTOM_GRAPH,      //< 自定义图形，请与自定义图形工具
  TEDU_BOARD_ELEMENT_AUDIO,             //< 音频元素，当前白板有效
  TEDU_BOARD_ELEMENT_GLOBAL_AUDIO,      //< 全局音频元素，无控制界面，一般为背景音乐
}

/// 位置类型
enum TEduBoardPosition {
  TEDU_BOARD_POSITION_LEFT_TOP,          ///< 左上方
  TEDU_BOARD_POSITION_RIGHT_TOP,         ///< 右上方
  TEDU_BOARD_POSITION_RIGHT_BOTTOM,      ///< 右下方
  TEDU_BOARD_POSITION_LEFT_BOTTOM,       ///< 左下方
}

enum AndroidPlatformViewMod {
  VIRTUAL_DISPLAYS,                       /// 虚拟显示模式
  HYBRID_COMPOSITION                      /// 混合集成模式
}

/// =============白板数据类型定义=============

/// 白板信息
class TEduBoardInfo {
  ///< 白板id
  String boardId;

  ///< 背景图或者背景H5的URL
  String backgroundUrl;

  ///< 白板背景色
  int backgroundColor;

  TEduBoardInfo(this.boardId, this.backgroundUrl, this.backgroundColor);
}

/// 文件信息
class TEduBoardFileInfo {
  ///< 文件标识
  String fileId;

  ///< 文件名称
  String title;

  ///<文件下载地址
  String downloadURL;

  ///< 当前页数
  int pageIndex;

  ///< 页数
  int pageCount;

  ///< 白板信息列表
  List<TEduBoardInfo> boardInfoList;

  TEduBoardFileInfo(this.fileId, this.title, this.downloadURL, this.pageIndex, this.pageCount, this.boardInfoList);
}

/// 元素信息
class TEduBoardElementInfo {
  ///< 白板id
  String boardId;

  ///< 元素id
  String elementId;

  ///< 元素类型
  TEduBoardElementType type;

  ///< 元素地址
  String url;

  TEduBoardElementInfo({this.boardId, this.elementId, this.type, this.url});

  @override
  String toString() {
    return "{boardId:${boardId ?? ''}, elementId:${elementId ?? ''}, type:${type ?? ''}}, url:${url ?? ''}";
  }
}

/// 授权参数
class TEduBoardAuthParam {
  ///< 应用标识
  int sdkAppId;

  ////< 用户标识
  String userId;

  ///< 签名
  String userSig;

  TEduBoardAuthParam(this.sdkAppId, this.userId, this.userSig);
}

class TEduBoardInitParam {
  ///<默认白板宽高比，可传格式如“4:3”、“16:9”的字符串，默认“16:9”
  String ratio;

  ///<是否允许涂鸦，默认 YES
  bool drawEnable;

  ///<全局背景色 hex 0xFFFFFFFF white
  int globalBackgroundColor;

  ///<白板工具，默认 TEDU_BOARD_TOOL_TYPE_PEN
  TEduBoardToolType toolType;

  ///<画笔颜色 hex 0xFFFF0000 red
  int brushColor;

  ///<画笔粗细，默认100
  int brushThin;

  ///<文本颜色 hex 0xFF000000 black
  int textColor;

  ///<文本粗细，默认320
  int textSize;

  ///<文本样式，默认 TEDU_BOARD_TEXT_STYLE_NORMAL
  TEduBoardTextStyle textStyle;

  ///<是否启用数据同步，禁用后将导致本地白板操作不会被同步给远端
  bool dataSyncEnable;

  ///<图片预加载深度，默认值5，表示预加载当前页前后5页的图片
  int preloadDepth;

  ///<平滑系数，默认0，范围[0，1]
  double smoothLevel;

  ///<白板内容自适应模式，默认 TEDU_BOARD_CONTENT_FIT_MODE_NONE
  TEduBoardContentFitMode boardContentFitMode;

  ///<图片加载超时时间，单位秒默认10s
  int imageTimeout;

  ///<启用加载图标，主要作用于图片加载、ppt 加载过程，默认为 false
  bool progressEnable;

  ///<自定义加载图标，在 progressEnable = true 时生效，支持 jpg、gif、png、svg
  String progressBarUrl;

  ///<是否启用原生系统光标，默认false
  bool systemCursorEnable;

  ///<是否启用白板缩放移动工具的缩放功能，默认为true
  bool enableScaleTool;

  AndroidPlatformViewMod androidPlatformViewMod;

  TEduBoardInitParam(
      {
        this.ratio = '16:9',
        this.drawEnable = true,
        this.globalBackgroundColor = 0xFFFFFFFF,
        this.toolType = TEduBoardToolType.TEDU_BOARD_TOOL_TYPE_PEN,
        this.brushColor = 0xFFFF0000,
        this.brushThin = 100,
        this.textColor = 0xFF000000,
        this.textSize = 320,
        this.textStyle = TEduBoardTextStyle.TEDU_BOARD_TEXT_STYLE_NORMAL,
        this.dataSyncEnable = true,
        this.preloadDepth = 5,
        this.smoothLevel = 0,
        this.boardContentFitMode =
            TEduBoardContentFitMode.TEDU_BOARD_CONTENT_FIT_MODE_NONE,
        this.imageTimeout = 10,
        this.progressEnable = false,
        this.progressBarUrl = '',
        this.systemCursorEnable = false,
        this.enableScaleTool = true,
        this.androidPlatformViewMod = AndroidPlatformViewMod.HYBRID_COMPOSITION
      }
  );
}

/// 直线样式
class TEduBoardLineStyle {
  ///<直线类型，默认 TEDU_BOARD_LINE_TYPE_SOLID
  TEduBoardLineType lineType;

  ///<起点箭头类型，默认 TEDU_BOARD_ARROW_TYPE_NONE
  TEduBoardArrowType startArrowType;

  ///<终点箭头类型，默认 TEDU_BOARD_ARROW_TYPE_NONE
  TEduBoardArrowType endArrowType;

  TEduBoardLineStyle( 
    {this.lineType = TEduBoardLineType.TEDU_BOARD_LINE_TYPE_SOLID,
    this.startArrowType = TEduBoardArrowType.TEDU_BOARD_ARROW_TYPE_NONE,
    this.endArrowType = TEduBoardArrowType.TEDU_BOARD_ARROW_TYPE_NONE});
  

  @override
  String toString() {
    return "{lineType:$lineType, startArrowType:$startArrowType, endArrowType:$endArrowType}";
  }
}

/// 文件转码参数
class TEduBoardTranscodeConfig {
  ///<指定转码结果最小分辨率，可用于提高转码结果清晰度，如960x540，宽高分隔符为x
  String minRosolution;

  ///<指定对于 PPT 文件是否启用静态转码（转为静态图片），默认转为 H5 动画（转码耗时较长）
  bool isStaticPPT;
  
  ///<指定文件生成缩略图分辨率，默认不生成缩略图（生成缩略图增加额外转码消耗），如200x200，宽高分隔符为x
  String thumbnailResolution;

  TEduBoardTranscodeConfig(
      {this.minRosolution = '',
      this.isStaticPPT = false,
      this.thumbnailResolution = ''});

}

/// 文件转码结果
class TEduBoardTranscodeFileResult {
  ///<任务 ID
  String taskId;
  
  ///<转码状态
  TEduBoardFileTranscodeStatus status;

  ///<转码进度（百分比）
  int progress;

  ///<文件标题
  String title;

  ///<文件分辨率，如@"1024x768"
  String resolution;

  ///<文件总页数
  int pages;

  ///<转码结果 URL
  String url;

  ///<文件生成缩略图的分辨率，如@"200x200"
  String thumbnailResolution;

  ///<文件生成缩略图 URL
  String thumbnailUrl;

  TEduBoardTranscodeFileResult(
    this.taskId, 
    this.status, 
    this.progress, 
    this.title, 
    this.resolution, 
    this.pages, 
    this.url,
    this.thumbnailResolution,
    this.thumbnailUrl);
}

/// 鼠标样式
class TEduBoardCursorIcon {
  ///<浏览器内置鼠标指针样式，[可取值参考文档](https://developer.mozilla.org/zh-CN/docs/Web/CSS/cursor)
  String cursor;

  ///<自定义鼠标指针样式的 URL，[格式限制参考文档](https://developer.mozilla.org/zh-CN/docs/Web/CSS/cursor/url)
  String url;

  ///<自定义鼠标指针样式的横行偏移
  int offsetX;

  ///<自定义鼠标指针样式的纵向偏移
  int offsetY;

  TEduBoardCursorIcon(
    {this.cursor = 'none',
    this.url,
    this.offsetX,
    this.offsetY});
}

/// 快照信息
class TEduBoardSnapshotInfo {
  ///<快照文件存储的本地路径，包括文件名和后缀，只支持 png，例如：aaa/bbb/ccc/xxx.png
  String path;

  TEduBoardSnapshotInfo(
    {this.path = ''});
}

/// 工具提示语样式
class TEduBoardToolTypeTitleStyle {
  ///< 显示位置
  TEduBoardPosition  position;

  ///< 文本颜色 hex
  int color;

  ///< 实际像素值取值(size * 白板的高度 / 10000)px
  int size;

  ///< 文本样式
  TEduBoardTextStyle style; 
}

/** 
/// =============白板事件回调接口=============

/// 通用事件回调 
/// @{

/// 白板错误回调
/// @param code 错误码
/// @param msg 错误信息
typedef void TEBErrorCallback(TEduBoardErrorCode code, String msg);

/// 白板警告回调
/// @param code 错误码
/// @param msg  错误信息
typedef void TEBWarningCallback(TEduBoardWarningCode code, String msg);

/// @}

/// 基本流程回调 
/// @{

/// 白板初始化完成回调
/// 收到该回调后表示白板已处于可正常工作状态（此时白板为空白白板，历史数据尚未拉取到）
typedef void TEBInitCallback();

/// 白板历史数据同步完成回调
typedef void TEBHistroyDataSyncCompletedCallback();

/// 白板同步数据回调
/// 收到该回调时需要将回调数据通过信令通道发送给房间内其他人，接受者收到后调用 addSyncData 接口将数据添加到白板以实现数据同步
/// 该回调用于多个白板间的数据同步
typedef void TEBSyncDataCallback(String data);

/// 白板可撤销状态改变回调
/// @param canUndo 白板当前是否还能执行 Undo 操作
typedef void TEBUndoStatusChangedCallback(bool canUndo);

/// 白板可重做状态改变回调
/// @param canRedo 白板当前是否还能执行 Redo 操作
typedef void TEBRedoStatusChangedCallback(bool canRedo);

/// @}

/// 涂鸦功能回调 
/// @{

/// 白板状态改变回调
/// @param boardId 白板 ID
/// @param url 白板图片 URL
/// @param status 新的白板图片状态
typedef void TEBImageStatusChangedCallback(String boardId, String url, TEduBoardImageStatus status);

// todo: to be continue

/// @}

/// 白板操作回调
/// @{

/// 刷新白板回调
typedef void TEBRefreshCallback();

// todo: to be continue

/// @}

/// 文件操作回调
/// @{

// todo: to be continue

/// @}

*/