import 'dart:convert';
import 'dart:io';
import 'dart:typed_data';
import 'package:path/path.dart';
import 'package:mime/mime.dart';

String rgbColor(int hex) {
  //0xFFEEFFOO
  double alpha = ((0xFF000000 & hex) >> 24) / 0xFF;
  int red = (0x00FF0000 & hex) >> 16;
  int green = (0x0000FF00 & hex) >> 8;
  int blue = (0x000000FF & hex) >> 0;
  return 'rgba($red, $green, $blue, $alpha)';
}

int hexColor(String rgba) {
  //rgba(252, 186, 3, 0.5058823529411764)
  if (rgba == null) {
    return null;
  }

  List<String> rgbas = rgba
      .replaceAll(RegExp(r'\s'), '')
      .replaceAll(RegExp(r'rgba\(|\)'), '')
      .split(',');

  int value = (((double.parse(rgbas[3]) * 0xff ~/ 1) & 0xff) << 24) |
      ((int.parse(rgbas[0]) & 0xff) << 16) |
      ((int.parse(rgbas[1]) & 0xff) << 8) |
      ((int.parse(rgbas[2]) & 0xff) << 0) & 0xFFFFFFFF;
  return value;
}

String eduStringEscape(String str) {
  return str
      .replaceAll('\\', '\\\\')
      .replaceAll('\"', '\\\"')
      .replaceAll("\'", "\\\'");
}

///本地文件上传信息 for js
Future<String> fileData(String path) async {
  File file = new File(path);
  if (!file.existsSync()) {
    return null;
  }

  //本地文件
  String fileName = Uri.encodeQueryComponent(basename(file.path));
  String mime = lookupMimeType(fileName);
  Uint8List bytes = await file.readAsBytes();
  String base64 = base64Encode(bytes);

  return "{data:'data:$mime;base64,$base64',name:'$fileName',userData:'$path'}";
}
